////////// Importeren Liberarys /////////
#include <Adafruit_GFX.h>     // Core graphics library
#include <Adafruit_ST7735.h>  // Hardware-specific library for ST773
#include <SPI.h>              // SPI library for comunication
#include <WiFi.h>             // WiFi library
#include <WiFiMulti.h>        // meerdere netwerken Wifi Library
#include <HTTPClient.h>       // For GET requests
#include <Arduino_JSON.h>     // For reciving a json object of openwheatermap


///////// Define TFT Display Pins /////////
#define TFT_CS 2
#define TFT_RST 22  // Or set to -1 and connect to Arduino RESET pin
#define TFT_DC 19
#define TFT_MOSI 21       // Data out
#define TFT_SCLK 18       // Clock out
#define TFT_BackLight 23  // Clock out


///////// Define Colors for TFT Screen /////////
#define BLACK 0x0000
#define BLUE 0x001F
#define RED 0xF800
#define GREEN 0x07E0
#define CYAN 0x07FF
#define MAGENTA 0xF81F
#define YELLOW 0xFFE0
#define WHITE 0xFFFF

// thermometer graphic coordinates and size
#define thermXTop 30       // thermometer tube top center x coordinate
#define thermYTop 112      // thermometer tube top center y coordinate
#define thermHeight 35     // thermometer tube height
#define thermTubeWidth 10  // thermometer tube width
#define thermBulbRadius 8  // thermometer bulb radius

#define interruptPin 12

bool gedrukkt = false;
/////////// NTP Client /////////
const char* ntpServer = "pool.ntp.org";
const long gmtOffset_sec = 0;
const int daylightOffset_sec = 3600;

// Variable om snel het display aan te spreken
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_MOSI, TFT_SCLK, TFT_RST);

// variabel om snel de multi WiFi aan te spreken
WiFiMulti wifiMulti;

// variabels for openwheathermap
String openWeatherMapApiKey = "Key";

// Replace with your country code and city
String city = "city";
String countryCode = "country";

const char* serverName = "http://192.168.68.239:5000/api/v1/log/data/";
////////// Variabels for clock /////////
float p = 3.1415926;
float angle;
int ccenterx, ccentery;  //center x,y of the clock
const int cradius = 40;  //radius of the clock
const float scosConst = 0.0174532925;
float sx = 0, sy = 1, mx = 1, my = 0, hx = -1, hy = 0;
float sdeg = 0, mdeg = 0, hdeg = 0;
int osx, osy, omx, omy, ohx, ohy;
int x0 = 0, x1 = 0, yy0 = 0, yy1 = 0;
long targetTime = 0;  // for next 1 second timeout
int hh, mm, ss;       //containers for current time

////////// Varibels for getting the wheather //////////
float temp_float = 0;
unsigned long lastTime = 0;
// Timer set to 10 minutes (600000)
unsigned long timerDelay = 600000;
// Set timer to 10 seconds (10000)
// unsigned long timerDelay = 10000;

///////// Functie for drawing the clocks face /////////
void drawClockFace() {
  tft.fillCircle(ccenterx, ccentery, cradius, CYAN);
  tft.fillCircle(ccenterx, ccentery, cradius - 4, BLACK);
  // Draw 12 lines
  for (int i = 0; i < 360; i += 30) {
    sx = cos((i - 90) * scosConst);
    sy = sin((i - 90) * scosConst);
    x0 = sx * (cradius - 4) + ccenterx;
    yy0 = sy * (cradius - 4) + ccentery;
    x1 = sx * (cradius - 11) + ccenterx;
    yy1 = sy * (cradius - 11) + ccentery;
    tft.drawLine(x0, yy0, x1, yy1, CYAN);
  }
}

///////// Functie for drawing the clocks hands /////////
void drawClockHands(int h, int m, int s) {
  // Pre-compute hand degrees, x & y coords for a fast screen update
  sdeg = s * 6;                      // 0-59 -> 0-354
  mdeg = m * 6 + sdeg * 0.01666667;  // 0-59 -> 0-360 - includes seconds
  hdeg = h * 30 + mdeg * 0.0833333;  // 0-11 -> 0-360 - includes minutes and seconds
  hx = cos((hdeg - 90) * scosConst);
  hy = sin((hdeg - 90) * scosConst);
  mx = cos((mdeg - 90) * scosConst);
  my = sin((mdeg - 90) * scosConst);
  sx = cos((sdeg - 90) * scosConst);
  sy = sin((sdeg - 90) * scosConst);

  // Erase just old hand positions
  tft.drawLine(ohx, ohy, ccenterx + 1, ccentery + 1, BLACK);
  tft.drawLine(omx, omy, ccenterx + 1, ccentery + 1, BLACK);
  tft.drawLine(osx, osy, ccenterx + 1, ccentery + 1, BLACK);
  // Draw new hand positions
  tft.drawLine(hx * (cradius - 28) + ccenterx + 1, hy * (cradius - 28) + ccentery + 1, ccenterx + 1, ccentery + 1, WHITE);
  tft.drawLine(mx * (cradius - 17) + ccenterx + 1, my * (cradius - 17) + ccentery + 1, ccenterx + 1, ccentery + 1, WHITE);
  tft.drawLine(sx * (cradius - 14) + ccenterx + 1, sy * (cradius - 14) + ccentery + 1, ccenterx + 1, ccentery + 1, RED);
  tft.fillCircle(ccenterx + 1, ccentery + 1, 3, RED);

  // Update old x&y coords
  osx = sx * (cradius - 14) + ccenterx + 1;
  osy = sy * (cradius - 14) + ccentery + 1;
  omx = mx * (cradius - 17) + ccenterx + 1;
  omy = my * (cradius - 17) + ccentery + 1;
  ohx = hx * (cradius - 28) + ccenterx + 1;
  ohy = hy * (cradius - 28) + ccentery + 1;
}

static uint8_t conv2d(const char* p) {
  uint8_t v = 0;
  if ('0' <= *p && *p <= '9') v = *p - '0';
  return 10 * v + *++p - '0';
}

void drawThermometer(void) {
  // draw thermometer tube outline
  tft.fillRoundRect((thermXTop - (thermTubeWidth / 2)), thermYTop - (thermTubeWidth / 2) + 1,
                    thermTubeWidth, thermHeight, thermTubeWidth / 2, WHITE);       // tube
  tft.fillCircle(thermXTop, (thermYTop + thermHeight), (thermBulbRadius), WHITE);  // bottom bulb

  // fill white thermometer tube inside
  tft.fillRoundRect((thermXTop - (thermTubeWidth / 2) + 3), thermYTop - (thermTubeWidth / 2) + 5,
                    (thermTubeWidth - 6), thermHeight, (thermTubeWidth / 2) - 0.5, BLACK);  // tube
  tft.fillCircle(thermXTop, (thermYTop + thermHeight), (thermBulbRadius - 3), BLACK);       // bottom bulb

  // fill red thermometer bulb
  tft.fillCircle(thermXTop, (thermYTop + thermHeight), (thermBulbRadius - 6), RED);

  // fill red tube to connect between lowest reading level and the bulb reservoir
  tft.fillRect((thermXTop - (thermTubeWidth / 2) + 6), thermYTop + thermHeight - thermBulbRadius, (thermTubeWidth - 12), 7, RED);

  tft.fillRect(thermXTop + (thermTubeWidth / 2) + 1, (thermYTop + (thermHeight - thermBulbRadius) * 0.80), 3, 2, WHITE);  // "20" level
  tft.fillRect(thermXTop + (thermTubeWidth / 2) + 1, (thermYTop + (thermHeight - thermBulbRadius) * 0.60), 3, 2, WHITE);  // "40" level
  tft.fillRect(thermXTop + (thermTubeWidth / 2) + 1, (thermYTop + (thermHeight - thermBulbRadius) * 0.40), 3, 2, WHITE);  // "60" level
  tft.fillRect(thermXTop + (thermTubeWidth / 2) + 1, (thermYTop + (thermHeight - thermBulbRadius) * 0.20), 3, 2, WHITE);  // "80" level
}



void printLocalTime() {

  struct tm timeinfo;
  if (!getLocalTime(&timeinfo)) {
    Serial.println("Failed to obtain time");
    return;
  }
  tft.setTextColor(ST7735_WHITE);
  tft.setTextSize(1);
  tft.setCursor(23, 5);
  tft.println(&timeinfo, "%d %B %Y");
  drawClockHands(timeinfo.tm_hour + 1, timeinfo.tm_min, timeinfo.tm_sec);
}

String getWheather() {
  // Send an HTTP GET request
  // Check WiFi connection status
  if (WiFi.status() == WL_CONNECTED) {
    String serverPath = "http://api.openweathermap.org/data/2.5/weather?q=" + city + "," + countryCode + "&APPID=" + openWeatherMapApiKey;

    String jsonBuffer = httpGETRequest(serverPath.c_str());
    // Serial.println(jsonBuffer);
    JSONVar myObject = JSON.parse(jsonBuffer);

    // JSON.typeof(jsonVar) can be used to get the type of the var
    if (JSON.typeof(myObject) == "undefined") {
      // Serial.println("Parsing input failed!");
      return ("Parsing input failed!");
    }

    // Serial.print("JSON object = ");
    // Serial.println(myObject);
    // Serial.print("Temperature: ");
    // Serial.println(myObject["main"]["temp"]);
    // Serial.print("Pressure: ");
    // Serial.println(myObject["main"]["pressure"]);
    // Serial.print("Humidity: ");
    // Serial.println(myObject["main"]["humidity"]);
    // Serial.print("Wind Speed: ");
    // Serial.println(myObject["wind"]["speed"]);
    return (JSON.stringify(myObject["main"]["temp"]));
  } else {
    Serial.println("WiFi Disconnected");
  }
}

String httpGETRequest(const char* serverName) {
  WiFiClient client;
  HTTPClient http;

  // Your Domain name with URL path or IP address with path
  http.begin(client, serverName);

  // Send HTTP POST request
  int httpResponseCode = http.GET();

  String payload = "{}";

  if (httpResponseCode > 0) {
    Serial.print("HTTP Response code: ");
    Serial.println(httpResponseCode);
    payload = http.getString();
  } else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }
  // Free resources
  http.end();

  return payload;
}

// void stuur_request() {
//       WiFiClient client;
//       HTTPClient http;
      
   
// }

void ingedrukt(){
  if (gedrukkt == false){
    gedrukkt = true;
    Serial.println("ingedrukt");
  }
  
}

void setup() {

  Serial.begin(115200);  // setup serial monitor
  delay(10);


  wifiMulti.addAP("SSID", "PASWORD");
  wifiMulti.addAP("SSID", "PASWORD");
  wifiMulti.addAP("SSID", "PASWORD");

  Serial.println("Connecting Wifi...");
  if (wifiMulti.run() == WL_CONNECTED) {
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
  }

  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);


  pinMode(TFT_BackLight, OUTPUT);
  digitalWrite(TFT_BackLight, 1);
  tft.initR(INITR_BLACKTAB);  // Init ST7735S chip, black tab
  Serial.println(F("Initialized"));

  tft.fillScreen(ST77XX_BLACK);
  tft.fillScreen(ST77XX_BLACK);
  tft.setTextColor(WHITE, BLACK);
  ccenterx = tft.width() / 2;
  ccentery = 120 / 2;
  osx = ccenterx;
  osy = ccentery;
  omx = ccenterx;
  omy = ccentery;
  ohx = ccenterx;
  ohy = ccentery;
  drawClockFace();  // Draw clock face
  printLocalTime();
  drawThermometer();
  String temp = getWheather();
  Serial.println(temp);
  temp_float = temp.toFloat();
  temp_float = temp_float - 273.15;
  tft.setTextColor(ST7735_WHITE);
  tft.setTextSize(2);
  tft.setCursor(50, 120);
  tft.print(temp_float, 1);
  // tft.setTextSize(1);
  // tft.setCursor(50, 125);
  tft.print(char(247));
  tft.print("C");
  tft.setTextSize(1);
  tft.setCursor(50, 140);
  tft.print("Kortrijk");

  pinMode(interruptPin, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(interruptPin), ingedrukt, FALLING);

}


void loop() {

  delay(1000);
  printLocalTime();
  if ((millis() - lastTime) > timerDelay) {
    tft.setTextColor(ST7735_BLACK);
    tft.setTextSize(2);
    tft.setCursor(50, 120);
    tft.print(temp_float, 1);
    tft.print(char(247));
    tft.print("C");
    String temp = getWheather();
    Serial.println(temp);
    temp_float = temp.toFloat();
    temp_float = temp_float - 273.15;
    tft.setTextColor(ST7735_WHITE);
    tft.setTextSize(2);
    tft.setCursor(50, 120);
    tft.print(temp_float, 1);
    tft.print(char(247));
    tft.print("C");
    lastTime = millis();
  }
  if (gedrukkt == true){
    gedrukkt = false;
    Serial.println('ingedrukt2');
    Serial.print("start ");
      WiFiClient client;
      HTTPClient http;
      // Your Domain name with URL path or IP address with path
      http.begin(client, serverName);
      // If you need an HTTP request with a content type: application/json, use the following:
      http.addHeader("Content-Type", "application/json");
      int httpResponseCode = http.POST("{\"id\":\"7\", \"value\":\"1\"}");
      Serial.print("HTTP Response code: ");
      Serial.println(httpResponseCode);
      // Free resources
      http.end();
    
    delay(10000);
  }
}